@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 544 $ : Last Committed Revision
- * $Date: 2010-06-23 15:45:12 -0700 (Wed, 23 Jun 2010) $ : Last Committed Date */
+ * $Revision: 561 $ : Last Committed Revision
+ * $Date: 2010-08-30 15:31:37 -0700 (Mon, 30 Aug 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -43,8 +43,14 @@
 #include <netdb.h>
 #include <pthread.h>
 #include "config.h"
+#if HAVE_LIBBLUETOOTH
+#include <bluetooth/bluetooth.h>
+#include <bluetooth/rfcomm.h>
+#include <Ws2bth.h>
+#endif
+
 #else
-#include <winsock.h>
+#include <winsock2.h>
 #include <windows.h>
 #include <time.h>
 #include "winconfig.h"
@@ -658,6 +664,10 @@ listen_Thread( LPVOID arg )
   int sockfd;
   struct sockaddr_in sktin;
   struct sockaddr_in peer_addr;
+#if HAVE_LIBBLUETOOTH
+  struct sockaddr_rc loc_addr = { 0 }, rem_addr = { 0 };
+  SOCKADDR_BTH loc_addr = { 0 }, rem_addr = { 0 };
+#endif
 #else
   SOCKET connectionsockfd;
   SOCKET sockfd;
@@ -685,23 +695,70 @@ listen_Thread( LPVOID arg )
   /* basic initialization */
   connection_number = 0;
 
-  connectionlen = sizeof(struct sockaddr_in);
+
+  if (mc_platform->bluetooth == 0) {
+    connectionlen = sizeof(struct sockaddr_in);
+  } else {
+#if HAVE_LIBBLUETOOTH
+#ifdef _WIN32
+    connectionlen = sizeof(SOCKADDR_BTH);
+#else
+    connectionlen = sizeof(struct sockaddr_rc);
+#endif
+#else
+    fprintf(stderr, "ERROR: This copy of Mobile-C was not compiled with Bluetooth support, but\n"
+     "bluetooth support was requested in the Mobile-C options. Please re-compile \n"
+     "Mobile-C with Bluetooth support.\n");
+    exit(-1);
+#endif
+  }
 
   /* Set up the socket */
-  sockfd = socket(PF_INET, SOCK_STREAM, 0);
+  if(mc_platform->bluetooth) {
+#if HAVE_LIBBLUETOOTH
+#ifndef _WIN32
+    sockfd = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
+#else
+    sockfd = socket(AF_BTH, SOCK_STREAM, BTHPROTO_RFCOMM); 
+#endif
+#endif
+  } else {
+    sockfd = socket(PF_INET, SOCK_STREAM, 0);
+  }
 	if (sockfd < 0) {
 		SOCKET_ERROR();
 	}
+
   mc_platform->sockfd = sockfd;
-  sktin.sin_family = AF_INET;
-  sktin.sin_port = htons(mc_platform->port);
-  sktin.sin_addr.s_addr = INADDR_ANY;
-  memset(sktin.sin_zero, '\0', sizeof sktin.sin_zero);
-  if (bind(sockfd, (struct sockaddr *)&sktin, sizeof(struct sockaddr))
-      == -1) {
-    fprintf(stderr, "bind() error. %s:%d\n",
-        __FILE__, __LINE__ );
-    exit(1);
+
+  if(mc_platform->bluetooth) {
+#if HAVE_LIBBLUETOOTH
+#ifndef _WIN32
+    loc_addr.rc_family = AF_BLUETOOTH;
+    loc_addr.rc_bdaddr = *BDADDR_ANY;
+    loc_addr.rc_channel = (uint8_t) mc_platform->agency->portno;
+#else
+    loc_addr.addressFamily= AF_BTH;
+    loc_addr.btAddr = 0;
+    loc_addr.port = mc_platform->agency->portno;
+#endif
+    if(bind(sockfd, (struct sockaddr *)&loc_addr, sizeof(loc_addr))) {
+      fprintf(stderr, "error binding to bluetooth socket. %s:%d\n", 
+        strerror(errno), errno);
+      exit(1);
+    }
+#endif
+  } else {
+    sktin.sin_family = AF_INET;
+    sktin.sin_port = htons(mc_platform->port);
+    sktin.sin_addr.s_addr = INADDR_ANY;
+    memset(sktin.sin_zero, '\0', sizeof sktin.sin_zero);
+    if (bind(sockfd, (struct sockaddr *)&sktin, sizeof(struct sockaddr))
+        == -1) {
+      fprintf(stderr, "bind() error. %s:%d\n",
+          __FILE__, __LINE__ );
+      exit(1);
+    }
   }
   listen(sockfd, BACKLOG);
 
@@ -713,74 +770,103 @@ listen_Thread( LPVOID arg )
     mc_platform->acc->waiting = 1;
     COND_BROADCAST(mc_platform->acc->waiting_cond);
     MUTEX_UNLOCK(mc_platform->acc->waiting_lock);
-#ifndef _WIN32
-    if((connectionsockfd = accept(sockfd, 
-            (struct sockaddr *)&peer_addr, 
-            (socklen_t *)&connectionlen)) < 0)
-#else
-      if((connectionsockfd = accept(sockfd,
-              (struct sockaddr *)&peer_addr,
-              (int*)&connectionlen)) == INVALID_SOCKET)
-#endif
-      {
-        fprintf(stderr, "ListenThread: accept error \n");
 #ifdef _WIN32
-        printf("Error number: %d\n", WSAGetLastError() );
-#endif
-        continue;
-      }
-      else 
-      {
-        /* Acquire host name of connected   */
-#ifndef _WIN32
-        getnameinfo((const struct sockaddr*)&peer_addr, sizeof(peer_addr),
-            peer_name, sizeof(peer_name), NULL, 0, 0);
-#else
-        addr.s_addr = inet_addr( inet_ntoa(peer_addr.sin_addr) );
-        if (addr.s_addr == INADDR_NONE) 
-          printf("The IPv4 address entered must be a legal address\n");
-        else
-          remoteHost = gethostbyaddr((char *) &addr, 4, AF_INET);
-
-        memset(peer_name, '\0', 45 );
-        strcpy(peer_name, remoteHost->h_name);
-
-#endif
-        //printf("peer_name = %s \n", peer_name);
-
-#ifdef NEW_SECURITY
-        /* Authentication Process for Received Connection  */
-        ret = auth_conn_rece_key(connectionsockfd, peer_name, &nonce, aes_key, mc_platform->private_key, mc_platform->agency->known_host_filename);
-        if( ret == 2 || ret == 1){
-          //printf("Successfully Authenticate %s \n", peer_name);
-          // Adding connection in connection_queue 
-#endif /* NEW_SECURITY */
-          /* Set waiting flag */
-          MUTEX_LOCK(mc_platform->acc->waiting_lock);
-          mc_platform->acc->waiting = 0;
-          COND_BROADCAST(mc_platform->acc->waiting_cond);
-          MUTEX_UNLOCK(mc_platform->acc->waiting_lock);
-          /* create a new connection and insert it into the connection linked 
-           * list */
-          connection = connection_New();
-          connection->connect_id = rand();
-          connection->remote_hostname = NULL;
-          connection->addr = peer_addr;
-          connection->serverfd = sockfd;
-          connection->clientfd = connectionsockfd;
-#ifdef NEW_SECURITY
-          connection->nonce = nonce;
-          connection->AES_key = aes_key;
-#endif /* NEW_SECURITY */
-
-          /* add the connection to list and increment the number of connections */
-          connection_queue_Add(mc_platform->connection_queue, connection);
-#ifdef NEW_SECURITY
-        }else{ 
-          printf("Unable to authenticate %s \n", peer_name);
+    if(mc_platform->bluetooth) {
+        fprintf(stderr, "Warning: Windows Mobile-C Bluetooth Agencies only support sending agents.\nReceiving agents is unsupported. Disabling ACC listen thread...\n");
+        while(1) {
+            Sleep(10000);
         }
+    }
+#endif
+    if(mc_platform->bluetooth == 0) {
+#ifndef _WIN32
+      if((connectionsockfd = accept(sockfd, 
+              (struct sockaddr *)&peer_addr, 
+              (socklen_t *)&connectionlen)) < 0)
+#else
+        if((connectionsockfd = accept(sockfd,
+                (struct sockaddr *)&peer_addr,
+                (int*)&connectionlen)) == INVALID_SOCKET)
+#endif
+        {
+          fprintf(stderr, "ListenThread: accept error \n");
+#ifdef _WIN32
+          printf("Error number: %d\n", WSAGetLastError() );
+#endif
+          continue;
+        }
+    } else {
+#if HAVE_LIBBLUETOOTH
+#ifndef _WIN32
+      if((connectionsockfd = accept(sockfd, 
+              (struct sockaddr *)&peer_addr, 
+              (socklen_t *)&connectionlen)) < 0)
+#else
+        if((connectionsockfd = accept(sockfd,
+                (struct sockaddr *)&rem_addr,
+                (int*)&connectionlen)) == INVALID_SOCKET)
+#endif
+        {
+          fprintf(stderr, "ListenThread: accept error \n");
+#ifdef _WIN32
+          printf("Error number: %d\n", WSAGetLastError() );
+#endif
+          continue;
+        }
+#endif
+    }
+    if(connectionsockfd > 0) 
+    {
+      /* Acquire host name of connected   */
+#ifndef _WIN32
+      getnameinfo((const struct sockaddr*)&peer_addr, sizeof(peer_addr),
+          peer_name, sizeof(peer_name), NULL, 0, 0);
+#else
+      addr.s_addr = inet_addr( inet_ntoa(peer_addr.sin_addr) );
+      if (addr.s_addr == INADDR_NONE) 
+        printf("The IPv4 address entered must be a legal address\n");
+      else
+        remoteHost = gethostbyaddr((char *) &addr, 4, AF_INET);
+
+      memset(peer_name, '\0', 45 );
+      strcpy(peer_name, remoteHost->h_name);
+
+#endif
+      //printf("peer_name = %s \n", peer_name);
+
+#ifdef NEW_SECURITY
+      /* Authentication Process for Received Connection  */
+      ret = auth_conn_rece_key(connectionsockfd, peer_name, &nonce, aes_key, mc_platform->private_key, mc_platform->agency->known_host_filename);
+      if( ret == 2 || ret == 1){
+        //printf("Successfully Authenticate %s \n", peer_name);
+        // Adding connection in connection_queue 
 #endif /* NEW_SECURITY */
+        /* Set waiting flag */
+        MUTEX_LOCK(mc_platform->acc->waiting_lock);
+        mc_platform->acc->waiting = 0;
+        COND_BROADCAST(mc_platform->acc->waiting_cond);
+        MUTEX_UNLOCK(mc_platform->acc->waiting_lock);
+        /* create a new connection and insert it into the connection linked 
+         * list */
+        connection = connection_New();
+        connection->connect_id = rand();
+        connection->remote_hostname = NULL;
+        connection->addr = peer_addr;
+        connection->serverfd = sockfd;
+        connection->clientfd = connectionsockfd;
+#ifdef NEW_SECURITY
+        connection->nonce = nonce;
+        connection->AES_key = aes_key;
+#endif /* NEW_SECURITY */
+
+        /* add the connection to list and increment the number of connections */
+        connection_queue_Add(mc_platform->connection_queue, connection);
+#ifdef NEW_SECURITY
+      }else{ 
+        printf("Unable to authenticate %s \n", peer_name);
       }
+#endif /* NEW_SECURITY */
+    }
   }
 
   /* Free the current thread */
