@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 519 $ : Last Committed Revision
- * $Date: 2010-06-11 14:26:03 -0700 (Fri, 11 Jun 2010) $ : Last Committed Date */
+ * $Revision: 560 $ : Last Committed Revision
+ * $Date: 2010-08-30 15:09:20 -0700 (Mon, 30 Aug 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -125,17 +125,7 @@ MC_WaitSignal
 #ifndef _MOBILEC_H_
 #define _MOBILEC_H_
 
-#ifdef _WIN32
-#include <windows.h>
-#endif
-
-#ifndef THREAD_T
-#ifdef _WIN32
-#define THREAD_T HANDLE
-#else
-#define THREAD_T pthread_t
-#endif
-#endif
+#include "macros.h"
 
 #ifdef WIN32
 	#ifdef _MC_DLL
@@ -237,6 +227,7 @@ typedef struct agency_s {
     int threads;                        /*!< flag which determines which threads to start */
     int enable_security;                /*!< Security flag */
     int stack_size[MC_THREAD_ALL];
+    int bluetooth;
 
 		char* priv_key_filename;
 		char* known_host_filename;
@@ -261,6 +252,7 @@ typedef struct MCAgencyOptions_s{
     char *known_host_filename;
     char *priv_key_filename;
     int initInterps;
+    int bluetooth; /* Startup in bluetooth mode */
     ChOptions_t* ch_options;
 } MCAgencyOptions_t;
 
