@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 529 $ : Last Committed Revision
- * $Date: 2010-06-16 16:00:50 -0700 (Wed, 16 Jun 2010) $ : Last Committed Date */
+ * $Revision: 560 $ : Last Committed Revision
+ * $Date: 2010-08-30 15:09:20 -0700 (Mon, 30 Aug 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -76,6 +76,9 @@ mc_platform_Initialize(MCAgency_t agency, ChOptions_t* ch_options)
     mc_platform->stack_size[i] = agency->stack_size[i];
   }
 
+  mc_platform->bluetooth = agency->bluetooth;
+  mc_platform->agency = agency;
+
   /* WINDOWS ONLY: Initialize WinSock */
 #ifdef _WIN32
   nret = WSAStartup(wVersionRequested, &(mc_platform->wsaData));
@@ -88,7 +91,7 @@ mc_platform_Initialize(MCAgency_t agency, ChOptions_t* ch_options)
 	exit(0);
   }
 #endif /* _WIN32 */
- 
+
   gethostname(hostname, DEFAULT_HOSTNAME_LENGTH);
   if (strlen(hostname) < 1) {
 	  strcpy(hostname, "localhost");
@@ -100,12 +103,14 @@ mc_platform_Initialize(MCAgency_t agency, ChOptions_t* ch_options)
 #else
 	  fprintf(stderr, "Fatal Error %s:%d\n", __FILE__, __LINE__);
 #endif
-	  exit(0);
+	  //exit(0);
   }
   mc_platform->hostname = (char*)malloc(sizeof(char)*DEFAULT_HOSTNAME_LENGTH);
   CHECK_NULL(mc_platform->hostname, agency->last_error = MC_ERR_MEMORY;return NULL);
 
-  if(localhost->h_name)
+  if(localhost == NULL) 
+    strcpy(mc_platform->hostname, "127.0.0.1"); 
+  else if(localhost->h_name)
     strcpy(mc_platform->hostname, localhost->h_name); 
   else
     // might always be 127.0.0.1 which is not good for ACL messaging on a network
