@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 545 $ : Last Committed Revision
- * $Date: 2010-08-04 13:20:13 -0700 (Wed, 04 Aug 2010) $ : Last Committed Date */
+ * $Revision: 563 $ : Last Committed Revision
+ * $Date: 2010-08-31 11:59:15 -0700 (Tue, 31 Aug 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -42,7 +42,13 @@
 #include <netdb.h>
 #include <errno.h>
 #include "config.h"
+#if HAVE_LIBBLUETOOTH
+#include <bluetooth/bluetooth.h>
+#include <bluetooth/rfcomm.h>
+#include <Ws2bth.h>
+#endif
 #else
+#include <winsock2.h>
 #include "winconfig.h"
 #endif
 #include <mxml.h>
@@ -171,24 +177,26 @@ message_InitializeFromAgent(
   message->target = strdup("ams");
 	message->sending_agent_name = strdup(agent->name);
   /* Set up message->addr */
-  buf = (char*)malloc
-    (
-     sizeof(char) * 
-     (strlen(message->to_address)+1)
-    );
-  CHECK_NULL(buf, exit(0););
-  strcpy(buf, message->to_address);
-  destination_host = strtok_r(buf, ":", &save_ptr);
-  destination_port_str = strtok_r(NULL, ":", &save_ptr);
-  destination_port = atoi(destination_port_str);
-  message->addr = (struct sockaddr_in*)malloc(sizeof(struct sockaddr_in));
-  if ((host = gethostbyname(destination_host))) {
-    memcpy(&(message->addr->sin_addr), host->h_addr, host->h_length);
-    message->addr->sin_port = htons(destination_port);
-  } else {
-    WARN("Host not found.");
+  if(mc_platform->bluetooth == 0) {
+    buf = (char*)malloc
+      (
+       sizeof(char) * 
+       (strlen(message->to_address)+1)
+      );
+    CHECK_NULL(buf, exit(0););
+    strcpy(buf, message->to_address);
+    destination_host = strtok_r(buf, ":", &save_ptr);
+    destination_port_str = strtok_r(NULL, ":", &save_ptr);
+    destination_port = atoi(destination_port_str);
+    message->addr = (struct sockaddr_in*)malloc(sizeof(struct sockaddr_in));
+    if ((host = gethostbyname(destination_host))) {
+      memcpy(&(message->addr->sin_addr), host->h_addr, host->h_length);
+      message->addr->sin_port = htons(destination_port);
+    } else {
+      WARN("Host not found.");
+    }
+    free(buf);
   }
-  free(buf);
   return MC_SUCCESS;
 }
 
@@ -589,6 +597,39 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
   return 0;
 }
 
+#ifdef _WIN32
+typedef struct bdaddr_s {
+  UINT8 b[6];
+} bdaddr_t;
+
+void baswap(bdaddr_t *dst, const bdaddr_t *src)
+{
+	register unsigned char *d = (unsigned char *) dst;
+	register const unsigned char *s = (const unsigned char *) src;
+	register int i;
+
+	for (i = 0; i < 6; i++)
+		d[i] = s[5-i];
+}
+
+int str2ba(const char *str, bdaddr_t *ba)
+{
+	UINT8 b[6];
+	const char *ptr = str;
+	int i;
+
+	for (i = 0; i < 6; i++) {
+		b[i] = (UINT8) strtol(ptr, NULL, 16);
+		if (i != 5 && !(ptr = strchr(ptr, ':')))
+			ptr = ":00:00:00:00:00";
+		ptr++;
+	}
+
+	baswap(ba, (bdaddr_t *) b);
+
+	return 0;
+}
+#endif
 
 #define MSG_THREAD_EXIT() \
 	free(arg); \
@@ -616,6 +657,10 @@ message_send_Thread( LPVOID arg )
 #ifndef _WIN32
   int skt;
   struct sockaddr_in sktin;
+#if HAVE_LIBBLUETOOTH
+  struct sockaddr_rc btsktin;
+  SOCKADDR_BTH btsktin;
+#endif
   struct addrinfo* myaddrinfo;
   struct addrinfo hint_addrinfo;
 #else
@@ -655,52 +700,99 @@ message_send_Thread( LPVOID arg )
     MSG_THREAD_EXIT();
   }
 
-  /* We need to split up the address into a hostname and port. */
-  buf = (char*)malloc(sizeof(char)*(strlen(message->to_address)+1));
-  strcpy(buf, message->to_address);
-  hostname = strtok_r(buf, ":", &saveptr);
-  sscanf( strtok_r(NULL, ":", &saveptr), "%d", &port );
+  if(mc_platform->bluetooth) {
+#if HAVE_LIBBLUETOOTH
+#ifndef _WIN32
+    if((skt = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM)) < 0) 
+    { 
+      fprintf(stderr, "Error - can't create socket\n");
+      MSG_THREAD_EXIT();
+    }
+    btsktin.rc_family = AF_BLUETOOTH;
+    buf = (char*)malloc(sizeof(char)*(strlen(message->to_address)+1));
+    strcpy(buf, message->to_address);
+    hostname = strtok_r(buf, " ", &saveptr);
+    sscanf( strtok_r(NULL, " ", &saveptr), "%d", &port );
+    btsktin.rc_channel = (uint8_t) port;
+    str2ba( hostname, &btsktin.rc_bdaddr );
+#else
+    if((skt = socket(AF_BTH, SOCK_STREAM, BTHPROTO_RFCOMM)) < 0) 
+    { 
+      fprintf(stderr, "Error - can't create socket\n");
+      MSG_THREAD_EXIT();
+    }
+    btsktin.addressFamily = AF_BTH;
+    buf = (char*)malloc(sizeof(char)*(strlen(message->to_address)+1));
+    strcpy(buf, message->to_address);
+    hostname = strtok_r(buf, " ", &saveptr);
+    sscanf( strtok_r(NULL, " ", &saveptr), "%d", &port );
+    btsktin.port =  port;
+    str2ba( hostname, (bdaddr_t*)&btsktin.btAddr );
+#endif
+#endif
+  } else {
+    /* We need to split up the address into a hostname and port. */
+    buf = (char*)malloc(sizeof(char)*(strlen(message->to_address)+1));
+    strcpy(buf, message->to_address);
+    hostname = strtok_r(buf, ":", &saveptr);
+    sscanf( strtok_r(NULL, ":", &saveptr), "%d", &port );
 
-  if((skt = socket(PF_INET, SOCK_STREAM, 0)) < 0) 
-  { 
-    fprintf(stderr, "Error - can't create socket\n");
-    MSG_THREAD_EXIT();
+    if((skt = socket(PF_INET, SOCK_STREAM, 0)) < 0) 
+    { 
+      fprintf(stderr, "Error - can't create socket\n");
+      MSG_THREAD_EXIT();
+    }
+
+    memset(&sktin, 0, sizeof(sktin));
+    sktin.sin_family = PF_INET;
+    sktin.sin_port = htons(port);
   }
 
-  memset(&sktin, 0, sizeof(sktin));
-  sktin.sin_family = PF_INET;
-  sktin.sin_port = htons(port);
-
+  if (mc_platform->bluetooth == 0) {
 #ifndef _WIN32
-  //if(gethostbyname_r(hostname, &host, hostbuf, hostbuf_len, &host_result, &errnum)) 
-  memset(&hint_addrinfo, 0, sizeof(hint_addrinfo));
-  hint_addrinfo.ai_family = AF_INET;
-  if( (errnum = getaddrinfo(hostname, NULL, &hint_addrinfo, &myaddrinfo)) == 0 )
+    //if(gethostbyname_r(hostname, &host, hostbuf, hostbuf_len, &host_result, &errnum)) 
+    memset(&hint_addrinfo, 0, sizeof(hint_addrinfo));
+    hint_addrinfo.ai_family = AF_INET;
+    if( (errnum = getaddrinfo(hostname, NULL, &hint_addrinfo, &myaddrinfo)) == 0 )
 #else
-  if(host_result = gethostbyname(hostname)) 
+      if(host_result = gethostbyname(hostname)) 
 #endif
-  {
+      {
 #ifdef _WIN32
-    host = *host_result;
-	memcpy(&sktin.sin_addr, host.h_addr, host.h_length);
+        host = *host_result;
+        memcpy(&sktin.sin_addr, host.h_addr, host.h_length);
 #else
-    ((struct sockaddr_in*)myaddrinfo->ai_addr)->sin_port = htons(port);
+        ((struct sockaddr_in*)myaddrinfo->ai_addr)->sin_port = htons(port);
 #endif
+      }
+      else if((sktin.sin_addr.s_addr = inet_addr(hostname)) < 0) 
+      {
+        fprintf(stderr, "Error - can't get host entry for %s\n", hostname);
+        free(buf);
+        MSG_THREAD_EXIT();
+      } 
   }
-  else if((sktin.sin_addr.s_addr = inet_addr(hostname)) < 0) 
-  {
-    fprintf(stderr, "Error - can't get host entry for %s\n", hostname);
-    free(buf);
-    MSG_THREAD_EXIT();
-  } 
 
-  while(
 #ifndef _WIN32
-		(connect(skt, myaddrinfo->ai_addr, sizeof(struct sockaddr)) < 0) &&
-#else
-        (connect(skt, (struct sockaddr *) &sktin, sizeof(sktin)) < 0) &&
+  if (mc_platform->bluetooth) {
+#if HAVE_LIBBLUETOOTH
+    errnum = connect(skt, (struct sockaddr *)&btsktin, sizeof(btsktin));
 #endif
-		(num_tries < max_num_tries)
+  } else {
+    errnum = connect(skt, myaddrinfo->ai_addr, sizeof(struct sockaddr));
+  }
+#else
+  if (mc_platform->bluetooth) {
+#if HAVE_LIBBLUETOOTH
+    errnum = connect(skt, (struct sockaddr *) &btsktin, sizeof(btsktin));
+#endif
+  } else {
+    errnum = connect(skt, (struct sockaddr *) &sktin, sizeof(sktin));
+  }
+#endif
+  num_tries = 0;
+  while(
+		(num_tries < max_num_tries) && (errnum < 0)
 		) {
 		printf("ERROR Socket Connect failed... errno:%s\n", strerror(errno));
 #ifndef _WIN32
@@ -709,6 +801,23 @@ message_send_Thread( LPVOID arg )
 		Sleep(100);
 #endif
 		num_tries++;
+#ifndef _WIN32
+    if (mc_platform->bluetooth) {
+#if HAVE_LIBBLUETOOTH
+      errnum = connect(skt, (struct sockaddr *)&btsktin, sizeof(btsktin));
+#endif
+    } else {
+      errnum = connect(skt, myaddrinfo->ai_addr, sizeof(struct sockaddr));
+    }
+#else
+    if (mc_platform->bluetooth) {
+#if HAVE_LIBBLUETOOTH
+      errnum = connect(skt, (struct sockaddr *) &btsktin, sizeof(btsktin));
+#endif
+    } else {
+      errnum = connect(skt, (struct sockaddr *) &sktin, sizeof(sktin));
+    }
+#endif
   }
 	if (num_tries == max_num_tries) {
     fprintf(stderr, "Error - can't connect to %s:%d\n",
