@@ -4,16 +4,18 @@
 #ifdef _WIN32
 #include "winsock2.h"
 #include <memory.h>
+#else
+#include <sys/socket.h>
 #endif
 
 
 //#define COM 1	// comments
 
-
+#ifdef OBS
 static void mystrncpy_binary(char *dest, char *src, int start_index, int length) /*{{{*/
 {
 
-        int i, j, len, counter=0; 
+        int i, j, len; 
         j=start_index; 
         len = strlen(src);
         memset(dest, '\0', strlen(dest) );
@@ -21,6 +23,7 @@ static void mystrncpy_binary(char *dest, char *src, int start_index, int length)
                 dest[i] = src[j];
         }       
 }/*}}}*/
+#endif // OBS
 
 static void separate_key_parts(char *key, char *N, char *E, char *D, char *P, char *Q, char *DP, char *DQ, char*QP) /*{{{*/
 {
@@ -114,7 +117,12 @@ int rsa_encryption(char *publickey, char *plaintext, char *ciphertext) /*{{{*/
         // Encrypting plaintext
         ilen = strlen(plaintext);
         mode = RSA_EN;
-        status = rsa_pkcs1_encrypt(&rsa, mode, ilen, plaintext, ciphertext );
+        status = rsa_pkcs1_encrypt(
+            &rsa, 
+            mode, 
+            ilen, 
+            (unsigned char*)plaintext, 
+            (unsigned char*)ciphertext );
 #ifdef COM
         printf( " . Encryption status = %d \n",status );
 #endif
@@ -128,8 +136,6 @@ int rsa_decryption(char *ciphertext, char *plaintext, char *privatekey) /*{{{*/
      int ret, olen, status, mode;
 
      char N[265], E[13], D[265], P[140], Q[140], DP[140], DQ[140], QP[140];
-     FILE* fptr;
-     int i;
 
      memset(N,'\0',265); memset(E,'\0',13); memset(D,'\0',265); memset(P,'\0',140);
      memset(Q,'\0',140); memset(DP,'\0',140); memset(DQ,'\0',140); memset(QP,'\0',140);
@@ -165,7 +171,12 @@ int rsa_decryption(char *ciphertext, char *plaintext, char *privatekey) /*{{{*/
 
         mode = RSA_DE;
 		
-        status = rsa_pkcs1_decrypt(&rsa, mode, &olen, ciphertext, plaintext );
+        status = rsa_pkcs1_decrypt(
+            &rsa, 
+            mode, 
+            &olen, 
+            (unsigned char*)ciphertext, 
+            (unsigned char*)plaintext );
 	plaintext[strlen(plaintext)] = '\0';
 #ifdef COM
         printf( " . Decryption status = %d \n",status );        
@@ -210,7 +221,7 @@ static int remove_nonce_from_MA(char *MA_file){ /*{{{*/
 	
 	FILE *fd;
 	struct stat stbuf;	
-	int i=0, len; 
+	int len; 
 	char *MA_buffer, *p;
 	int numbytes;
 	
@@ -255,7 +266,6 @@ static int extract_nonce_from_MA(int sockfd, int *my_nonce, char *MA_file) /*{{{
 	char nonce_str[15];
 	FILE *fd;
 	struct stat stbuf;	
-	int i=0; //, len; 
 	int rece_nonce;
 	char ack[4];
 	int numbytes;
@@ -318,71 +328,71 @@ static int extract_nonce_from_MA(int sockfd, int *my_nonce, char *MA_file) /*{{{
 
 int read_known_host_file(char *pubkey, char *hname, char *filename)/*{{{*/
 {
-        FILE *fd;
-        char c;
-        char hostname[40];
-        int first,second,i,j;
-        int found=-1; 
-	char temp_E[15];
-        memset(pubkey,'\0', 300); 
-	memset(temp_E, '\0', 15);
-	second = i = j= 0;
-        first = 1;
+  FILE *fd;
+  char c;
+  char hostname[40];
+  int first,second,i,j;
+  int found=-1; 
+  char temp_E[15];
+  memset(pubkey,'\0', 300); 
+  memset(temp_E, '\0', 15);
+  second = i = j= 0;
+  first = 1;
 #ifdef COM
-		printf("Known host file lookup for peer %s's public key ... \n", hname);
+  printf("Known host file lookup for peer %s's public key ... \n", hname);
 #endif
 
-        if ( (fd = fopen (filename,"r"))== NULL ) {
-			printf("fopen() error. %s:%d\n" __FILE__, __LINE__);
-		return -1;
-        }
-        while(  (c=fgetc(fd)) != EOF ){
-                if(c =='#' ){
-                        pubkey[j]='\0';
-			first = 1; second = 0;
-                        i = j = 0;
-                        c = fgetc(fd);
-			if(c == ' ')
-			   while(fgetc(fd)==' '); // to avoid any space
-                        if (found == 1)
-                                break;
-                }
-                else if( second ){
-                //        pubkey[j++] = c;
-                }
-                else if(c == ' ' && first){
-                        first = 0; second = 1;
-                        hostname[i]='\0';
-                        if ( !strcmp(hostname, hname) ){
-                            found=1;
-			    for (i=0; i<260; i++) // reading N
-				pubkey[j++]= fgetc(fd);
-			    pubkey[j]='\n';
-			    while(fgetc(fd)!='\n'); // to avoid any space 
-			    j++;	
-			    for (i=0; i<10; i++)    // reading E	    
-				pubkey[j++]=fgetc(fd);
-			    pubkey[j]='\n';					
-			  //printf("%s's Pub key = '%s' size = %d \n",hostname,pubkey, strlen(pubkey));	
-				break;
-                        }else{
-			    memset(hostname, '\0', 40);
-			    memset(pubkey, '\0', 300);
-			 }
-                 }
-		else if(first && c!=' ' && c !='\r' && c!='\n'){
-                        hostname[i++] = c;
-                }
-	} // end while
+  if ( (fd = fopen (filename,"r"))== NULL ) {
+    printf("fopen() error. %s:%d\n", __FILE__, __LINE__);
+    return -1;
+  }
+  while(  (c=fgetc(fd)) != EOF ){
+    if(c =='#' ){
+      pubkey[j]='\0';
+      first = 1; second = 0;
+      i = j = 0;
+      c = fgetc(fd);
+      if(c == ' ')
+        while(fgetc(fd)==' '); // to avoid any space
+      if (found == 1)
+        break;
+    }
+    else if( second ){
+      //        pubkey[j++] = c;
+    }
+    else if(c == ' ' && first){
+      first = 0; second = 1;
+      hostname[i]='\0';
+      if ( !strcmp(hostname, hname) ){
+        found=1;
+        for (i=0; i<260; i++) // reading N
+          pubkey[j++]= fgetc(fd);
+        pubkey[j]='\n';
+        while(fgetc(fd)!='\n'); // to avoid any space 
+        j++;	
+        for (i=0; i<10; i++)    // reading E	    
+          pubkey[j++]=fgetc(fd);
+        pubkey[j]='\n';					
+        //printf("%s's Pub key = '%s' size = %d \n",hostname,pubkey, strlen(pubkey));	
+        break;
+      }else{
+        memset(hostname, '\0', 40);
+        memset(pubkey, '\0', 300);
+      }
+    }
+    else if(first && c!=' ' && c !='\r' && c!='\n'){
+      hostname[i++] = c;
+    }
+  } // end while
 
-        fclose(fd);
+  fclose(fd);
 #ifdef COM
-	printf("read_known_host_file(): Public key of connected peer : %s \n", pubkey);
+  printf("read_known_host_file(): Public key of connected peer : %s \n", pubkey);
 #endif
 
-	if(found == -1)
-		pubkey=NULL;
-	return found;
+  if(found == -1)
+    pubkey=NULL;
+  return found;
 }/*}}}*/
 
 int read_encrypted_file(char *enfile, char *string, unsigned char *passphase) /*{{{*/
@@ -472,7 +482,7 @@ int initiate_migration_process(int new_fd, int *my_nonce, char *pubkey, char *pr
 	printf("Send at socket %d \n", new_fd);
 #endif
 	generate_AES_key(my_challengetext);			
-	strcpy(aes_key, my_challengetext);
+	strcpy((char*)aes_key, my_challengetext);
 	
 #ifdef COM	
 	printf("AES key = %s size %d \n",aes_key, strlen(aes_key) );
@@ -552,7 +562,7 @@ int initiate_migration_process(int new_fd, int *my_nonce, char *pubkey, char *pr
        //printf("recev message %s\n", buffer);	
 #else
   numbytes = recvfrom(new_fd,
-      (void *) ciphertext,
+      ciphertext,
       (size_t) 135, /*sizeof(char)*SOCKET_INPUT_SIZE,*/
       0,
       (struct sockaddr *) 0,
@@ -700,7 +710,7 @@ int initiate_migration_process(int new_fd, int *my_nonce, char *pubkey, char *pr
       //printf("recev message %s\n", buffer);	
 #else
   numbytes = recvfrom(new_fd,
-      (void *) ciphertext,
+      ciphertext,
       (size_t) 135, /*sizeof(char)*SOCKET_INPUT_SIZE,*/
       0,
       (struct sockaddr *) 0,
@@ -824,7 +834,7 @@ int reply_migration_process(int sockfd, int *my_nonce, char *pubkey, char *privk
       //printf("recev message %s\n", buffer);	
 #else
   numbytes = recvfrom(sockfd,
-      (void *) ciphertext,
+      ciphertext,
       (size_t) 135, /*sizeof(char)*SOCKET_INPUT_SIZE,*/
       0,
       (struct sockaddr *) 0,
@@ -858,7 +868,7 @@ int reply_migration_process(int sockfd, int *my_nonce, char *pubkey, char *privk
         // separating rece_challenge from rece_plaintext( 0 - 31 )
 	for(i=0; i<32; i++)
 		rece_challengetext[i] = rece_plaintext[i];
-	strcpy(aes_key, rece_challengetext);
+	strcpy((char*)aes_key, rece_challengetext);
 #ifdef COM
 	printf("AES key = %s size %d \n",aes_key, strlen(aes_key) );
 #endif
@@ -967,7 +977,7 @@ int reply_migration_process(int sockfd, int *my_nonce, char *pubkey, char *privk
       0);
 #else
   numbytes = recvfrom(sockfd,
-      (void *) ciphertext,
+      ciphertext,
       (size_t) 135, /*sizeof(char)*SOCKET_INPUT_SIZE,*/
       0,
       (struct sockaddr *) 0,
@@ -1114,7 +1124,6 @@ int aes_en_de(int mode, char *infile, char *outfile, unsigned char *AES_key, int
     int ret = -1, i, n;
     int keylen, lastn;
 	FILE *fin, *fout;
-    int num=0;
     int flag= -1;  	
     char *p;
     unsigned char IV[16];
@@ -1165,7 +1174,7 @@ int aes_en_de(int mode, char *infile, char *outfile, unsigned char *AES_key, int
         return -1;
     }
 
-            keylen = strlen( AES_key );
+            keylen = strlen( (const char*)AES_key );
             
             if( keylen > (int) sizeof( key ) )
                 keylen = (int) sizeof( key );
@@ -1513,7 +1522,7 @@ int send_AES_en_MA(int sockfd, int *my_nonce ,char *outfile, char *pubkey) /*{{{
 
 #else
   numbytes = recvfrom(sockfd,
-      (void *) data,
+      data,
       (size_t) 20,
       0,
       (struct sockaddr *) 0,
@@ -1582,7 +1591,7 @@ int receive_AES_en_MA(int new_fd, int *nonce, char *infile, char *privkey) /*{{{
        0);
  #else
   numbytes = recvfrom(new_fd,
-      (void *) buffer,
+      buffer,
       (size_t) 135, 
       0,
       (struct sockaddr *) 0,
@@ -1646,7 +1655,7 @@ int receive_AES_en_MA(int new_fd, int *nonce, char *infile, char *privkey) /*{{{
       0);
 #else
   numbytes = recvfrom(new_fd,
-      (void *) buffer,
+      buffer,
       (size_t) size,
       0,
       (struct sockaddr *) 0,
